package com.codegeist.fisheye.internal.service;

import org.junit.Before;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

import junit.framework.TestCase;

public abstract class AbstractHandlerTest<T extends ContentHandler> extends TestCase {

	protected T handler;

	@Before
	protected final void setUp() throws Exception {
		handler = createHandler();
	}
	
	protected abstract T createHandler();
	
	protected void startDocument() throws SAXException {
		handler.startDocument();
	}
	
	protected void endDocument() throws SAXException {
		handler.endDocument();
	}
	
	protected void startElement(String element) throws SAXException {
		startElement(element, new String[0], new String[0]);
	}
	
	protected void startElement(String element, String[] names, String[] values) throws SAXException {
		AttributesImpl attributes = new AttributesImpl();
		for (int i = 0; i < names.length; i++) {
			String name = names[i];
			String value = values[i];
			
			attributes.addAttribute(null, name, name, "xs:string", value);
		}
		
		handler.startElement(null, element, element, attributes);
	}
	
	protected void endElement(String element) throws SAXException {
		handler.endElement(null, element, element);
	}
	
	protected void characters(String value) throws SAXException {
		handler.characters(value.toCharArray(), 0, value.length());
	}
}
