package com.codegeist.fisheye.internal.service;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import com.codegeist.fisheye.domain.Changesets;

public class ChangesetsHandler extends DefaultHandler implements Handler<Changesets> {
	private StringBuilder buffer = new StringBuilder();
	private Changesets result;
	private int maxReturn;
	private StringArrayHandler stringArrayHandler;
	private boolean inCsids;
	
	@Override
	public void startDocument() throws SAXException {
		maxReturn = 0;
		stringArrayHandler = new StringArrayHandler();
		inCsids = false;
	}
	

	@Override
	public void startElement(String uri, String localName, String name, Attributes attributes) throws SAXException {
		buffer.setLength(0);
		if (inCsids) {
			stringArrayHandler.startElement(uri, localName, name, attributes);
		} else {
			if ("changesets".equals(localName)) {
				maxReturn = parseInt(attributes.getValue("maxReturn"));
			} else if ("csids".equals(localName)) {
				inCsids = true;
				stringArrayHandler.startDocument();
			}
		}
	}
	
	@Override
	public void endElement(String uri, String localName, String name) throws SAXException {
		if (inCsids) {
			if ("csids".equals(localName)) {
				inCsids = false;
				stringArrayHandler.endDocument();
			} else {
				stringArrayHandler.endElement(uri, localName, name);
			}
		}
		
		buffer.setLength(0);
	}
	
	
	@Override
	public void endDocument() throws SAXException {
		result = new Changesets(maxReturn, stringArrayHandler.getResult());
	}
	
	@Override
	public void characters(char[] ch, int start, int length) throws SAXException {
		if (inCsids) {
			stringArrayHandler.characters(ch, start, length);
		} else {
			buffer.append(ch, start, length);
		}
	}
	
	public Changesets getResult() {
		return result;
	}
	
	private int parseInt(String value) throws SAXException {
		try {
			return Integer.parseInt(value);
		} catch (NumberFormatException e) {
			// TODO fix this
			throw new SAXException("ZOMG");
		}
	}
}
