package com.codegeist.fisheye.internal.ui.history;

import java.text.DateFormat;

import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.team.ui.history.HistoryPage;

import com.codegeist.fisheye.domain.PathHistory;
import com.codegeist.fisheye.domain.Revision;
import com.codegeist.fisheye.internal.ui.connection.model.PathModel;
import com.codegeist.fisheye.service.FisheyeException;

public class FisheyeHistoryPage extends HistoryPage {

	private TableViewer viewer;
	private DateFormat dateFormat = DateFormat.getDateTimeInstance();

	@Override
	public boolean inputSet() {
		PathModel input = (PathModel) getInput();
		viewer.setInput(input);
		
		return true;
	}

	@Override
	public void createControl(Composite parent) {
		viewer = new TableViewer(parent, SWT.H_SCROLL | SWT.V_SCROLL | SWT.MULTI | SWT.FULL_SELECTION);
		getSite().setSelectionProvider(viewer);
		viewer.getTable().setHeaderVisible(true);
		viewer.getTable().setLinesVisible(true);
		
		viewer.setContentProvider(new IStructuredContentProvider() {
		
			public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
			}
		
			public void dispose() {
			}
		
			public Object[] getElements(Object inputElement) {
				PathModel path = (PathModel) inputElement;
				try {
					PathHistory pathHistory = path.getRepo().getConnection().getService().listPathHistory(null, path.getRepo().getName(), path.getPathInfo().getAbsolutePath());
					
					return pathHistory.getRevisions();
				} catch (FisheyeException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				
				return null;
			}
		});
		
		TableViewerColumn version = new TableViewerColumn(viewer, SWT.NONE);
		version.getColumn().setWidth(75);
		version.getColumn().setText("Version");
		version.getColumn().setAlignment(SWT.RIGHT);
		
		version.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				Revision revision = (Revision) element;
				return revision.getKey().getRev();
			}
		});
		
		TableViewerColumn date = new TableViewerColumn(viewer, SWT.NONE);
		date.getColumn().setWidth(100);
		date.getColumn().setText("Date");
		date.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				Revision revision = (Revision) element;
				return dateFormat.format(revision.getDate());
			}
		});
		
		TableViewerColumn author = new TableViewerColumn(viewer, SWT.NONE);
		author.getColumn().setWidth(100);
		author.getColumn().setText("Author");
		author.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				Revision revision = (Revision) element;
				return revision.getAuthor();
			}
		});
		
		TableViewerColumn comment = new TableViewerColumn(viewer, SWT.NONE);
		comment.getColumn().setWidth(500);
		comment.getColumn().setText("Comment");
		comment.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				Revision revision = (Revision) element;
				String log = revision.getLog();
				if (log.indexOf('\n') != -1) {
					log = log.substring(0, log.indexOf('\n'));
				}
				return log;
			}
		});
	}

	@Override
	public Control getControl() {
		return viewer.getControl();
	}

	@Override
	public void setFocus() {
		viewer.getControl().setFocus();
	}

	public String getDescription() {
		PathModel pathModel = (PathModel) getInput();
		return "History of " + pathModel.getPathInfo().getAbsolutePath() + " (" + pathModel.getRepo().getName() + ")";
	}

	public String getName() {
		PathModel pathModel = (PathModel) getInput();
		return pathModel.getPathInfo().getAbsolutePath();
	}

	public boolean isValidInput(Object object) {
		return object instanceof PathModel;
	}

	public void refresh() {
	}

	public Object getAdapter(Class adapter) {
		return null;
	}

}
